/**
 * IMPORTANT !
 * Décommenter temporairement la ligne suivante (import { } from "./mock_jasmine")
 * Pour exécuter ce code dans le débugger.
 * Faire de même avec le fichier test_func.ts
 */
// import { describe, expect, it, xdescribe, xit } from "../mock_jasmine";

import { MessageCode } from "../../src";
import { ParamCalculability } from "../../src/param/param-definition";
import { Structure, StructureFlowMode, StructureFlowRegime, StructureParams } from "../../src/structure/structure";
import { Result } from "../../src/util/result";
import { checkResult } from "../test_func";

class StructureTest extends Structure {

    constructor(prms: StructureParams, dbg: boolean = false) {
        super(prms, dbg);
    }

    /**
     * Test of getFlowMode
     */
    public testGetFlowMode() {
        this.prms.update_h1h2();
        return this.getFlowMode();
    }

    /**
     * Test of getFlowRegime
     */
    public testGetFlowRegime() {
        this.prms.update_h1h2();
        return this.getFlowRegime();
    }

    public Equation(sVarCalc: string): Result {
        this.prms.update_h1h2();
        Structure.CheckEquation(sVarCalc);
        const data = this.getResultData();

        return new Result(this.prms.Z1.v - this.prms.Z2.v, data);
    }

    /**
     * Calcul de l'aire d'écoulement sur le seuil ou dans l'orifice
     */
    public calcA(): number {
        return Math.min(Math.max(this.prms.h1.v, this.prms.h2.v), this.prms.W.v);
    }

}

/* Test structure with :
 *  - ZDV = 0
 *  - Z1 = 30
 *  - Z2 = 15
 *  - expected Q = 15
 */
export const structTestPrm: StructureParams = new StructureParams(1, 0, 30, 15);
export const structTest: StructureTest = new StructureTest(structTestPrm, false);

/**
 * Test de calcul de tous les paramètres d'une loi de débit
 * @param st Structure à tester
 * @param bNotZDV Loi de débit ne permettant pas de calculer ZDV
 */
export function testStructure(
    st: Structure,
    mode: StructureFlowMode,
    regime: StructureFlowRegime,
    bNotZDV: boolean = false
) {
    for (const prm of st.prms) {
        if ([ParamCalculability.DICHO, ParamCalculability.EQUATION].includes(prm.calculability)) {
            if (prm.symbol === "W" && prm.v === Infinity) {
                // Le calcul de l'ouverture sur les seuils doit renvoyer une exception (cas impossible)
                it(`Calc(${prm.symbol}) should return exception`, () => {
                    expect(
                        () => { st.Calc(prm.symbol); }
                    ).toThrow(new Error("Structure:Calc : Calcul de W impossible sur un seuil"));
                });
            } else {
                const ref: number = prm.v;
                const res: Result = st.Calc(prm.symbol);
                if (bNotZDV) {
                    // Les lois CEM88D et CUNGE80 ne font pas intervenir ZDV dans le calcul d'un orifice noyé
                    it(`Calc(${prm.symbol}) should return an error`, () => {
                        expect(
                            st.Calc(prm.symbol).code
                        ).toBe(MessageCode.ERROR_STRUCTURE_ZDV_PAS_CALCULABLE);
                    });
                } else {
                    // On ne teste pas le calcul de l'ouverture sur les seuils
                    it(`Calc(${prm.symbol}) should return ${ref}`, () => {
                        checkResult(res, ref);
                    });
                }
                prm.v = ref; // Go back to initial value for following tests
            }
        }
    }
}